# """Unit tests configuration module."""

# import os
# import pytest
# from lmnr.traceloop_sdk import Traceloop
# from lmnr.traceloop_sdk.instruments import Instruments
# from lmnr.traceloop_sdk.tracing.tracing import TracerWrapper
# from opentelemetry.sdk.trace.export import SimpleSpanProcessor
# from opentelemetry.sdk.trace.export.in_memory_span_exporter import InMemorySpanExporter

# # pytest_plugins = []


# @pytest.fixture(scope="session")
# def exporter():
#     exporter = InMemorySpanExporter()
#     Traceloop.init(
#         app_name="test",
#         resource_attributes={"something": "yes"},
#         disable_batch=True,
#         exporter=exporter,
#     )
#     return exporter


# @pytest.fixture(autouse=True)
# def clear_exporter(exporter):
#     exporter.clear()


# @pytest.fixture(autouse=True)
# def environment():
#     if "OPENAI_API_KEY" not in os.environ:
#         os.environ["OPENAI_API_KEY"] = "test_api_key"


# @pytest.fixture(scope="module")
# def vcr_config():
#     return {
#         "filter_headers": ["authorization"],
#         "ignore_hosts": ["openaipublic.blob.core.windows.net"],
#     }


# @pytest.fixture
# def exporter_with_custom_span_processor():
#     # Clear singleton if existed
#     if hasattr(TracerWrapper, "instance"):
#         _trace_wrapper_instance = TracerWrapper.instance
#         del TracerWrapper.instance

#     class CustomSpanProcessor(SimpleSpanProcessor):
#         def on_start(self, span, parent_context=None):
#             span.set_attribute("custom_span", "yes")

#     exporter = InMemorySpanExporter()
#     Traceloop.init(
#         exporter=exporter,
#         processor=CustomSpanProcessor(exporter),
#     )

#     yield exporter

#     # Restore singleton if any
#     if _trace_wrapper_instance:
#         TracerWrapper.instance = _trace_wrapper_instance


# @pytest.fixture
# def exporter_with_custom_instrumentations():
#     # Clear singleton if existed
#     if hasattr(TracerWrapper, "instance"):
#         _trace_wrapper_instance = TracerWrapper.instance
#         del TracerWrapper.instance

#     exporter = InMemorySpanExporter()
#     Traceloop.init(
#         exporter=exporter,
#         disable_batch=True,
#         instruments=[i for i in Instruments],
#     )

#     yield exporter

#     # Restore singleton if any
#     if _trace_wrapper_instance:
#         TracerWrapper.instance = _trace_wrapper_instance


# @pytest.fixture
# def exporter_with_no_metrics():
#     # Clear singleton if existed
#     if hasattr(TracerWrapper, "instance"):
#         _trace_wrapper_instance = TracerWrapper.instance
#         del TracerWrapper.instance

#     os.environ["TRACELOOP_METRICS_ENABLED"] = "false"

#     exporter = InMemorySpanExporter()

#     Traceloop.init(
#         exporter=exporter,
#         disable_batch=True,
#     )

#     yield exporter

#     # Restore singleton if any
#     if _trace_wrapper_instance:
#         TracerWrapper.instance = _trace_wrapper_instance
#         os.environ["TRACELOOP_METRICS_ENABLED"] = "true"



# from opentelemetry.semconv_ai import SpanAttributes
# import pytest
# from openai import OpenAI
# from lmnr.traceloop_sdk.tracing.manual import LLMMessage, track_llm_call


# @pytest.fixture
# def openai_client():
#     return OpenAI()


# @pytest.mark.vcr
# def test_manual_report(exporter, openai_client):
#     with track_llm_call(vendor="openai", type="chat") as span:
#         span.report_request(
#             model="gpt-3.5-turbo",
#             messages=[
#                 LLMMessage(role="user", content="Tell me a joke about opentelemetry")
#             ],
#         )

#         res = openai_client.chat.completions.create(
#             model="gpt-3.5-turbo",
#             messages=[
#                 {"role": "user", "content": "Tell me a joke about opentelemetry"}
#             ],
#         )

#         span.report_response(res.model, [text.message.content for text in res.choices])

#     spans = exporter.get_finished_spans()
#     open_ai_span = spans[0]
#     assert open_ai_span.attributes[SpanAttributes.LLM_REQUEST_MODEL] == "gpt-3.5-turbo"
#     assert open_ai_span.attributes[f"{SpanAttributes.LLM_PROMPTS}.0.role"] == "user"
#     assert (
#         open_ai_span.attributes[f"{SpanAttributes.LLM_PROMPTS}.0.content"]
#         == "Tell me a joke about opentelemetry"
#     )
#     assert (
#         open_ai_span.attributes[SpanAttributes.LLM_RESPONSE_MODEL]
#         == "gpt-3.5-turbo-0125"
#     )
#     assert (
#         open_ai_span.attributes[f"{SpanAttributes.LLM_COMPLETIONS}.0.content"]
#         == "Why did the opentelemetry developer break up with their partner? Because they were tired"
#         + " of constantly tracing their every move!"
#     )
#     assert open_ai_span.end_time > open_ai_span.start_time



from enum import Enum
from opentelemetry.semconv_ai import SpanAttributes

SPAN_INPUT = "lmnr.span.input"
SPAN_OUTPUT = "lmnr.span.output"
SPAN_TYPE = "lmnr.span.type"
SPAN_PATH = "lmnr.span.path"

ASSOCIATION_PROPERTIES = "lmnr.association.properties"
SESSION_ID = "session_id"
USER_ID = "user_id"
TRACE_TYPE = "trace_type"


# exposed to the user, configurable
class Attributes(Enum):
    # == This is the minimum set of attributes for a proper LLM span ==
    #
    # not SpanAttributes.LLM_USAGE_PROMPT_TOKENS,
    INPUT_TOKEN_COUNT = "gen_ai.usage.input_tokens"
    # not SpanAttributes.LLM_USAGE_COMPLETION_TOKENS,
    OUTPUT_TOKEN_COUNT = "gen_ai.usage.output_tokens"
    TOTAL_TOKEN_COUNT = SpanAttributes.LLM_USAGE_TOTAL_TOKENS
    PROVIDER = SpanAttributes.LLM_SYSTEM
    REQUEST_MODEL = SpanAttributes.LLM_REQUEST_MODEL
    RESPONSE_MODEL = SpanAttributes.LLM_RESPONSE_MODEL
    #
    ## == End of minimum set ==
    # == Additional attributes ==
    #
    INPUT_COST = "gen_ai.usage.input_cost"
    OUTPUT_COST = "gen_ai.usage.output_cost"
    TOTAL_COST = "gen_ai.usage.cost"
    #
    # == End of additional attributes ==



# from opentelemetry.semconv_ai import SpanAttributes
# from lmnr.traceloop_sdk.decorators import task, workflow
# from pytest import raises


# def test_nested_tasks(exporter):
#     @workflow(name="some_workflow")
#     def some_workflow():
#         return outer_task()

#     @task(name="outer_task")
#     def outer_task():
#         return inner_task()

#     @task(name="inner_task")
#     def inner_task():
#         return inner_inner_task()

#     @task(name="inner_inner_task")
#     def inner_inner_task():
#         return

#     some_workflow()

#     spans = exporter.get_finished_spans()
#     assert [span.name for span in spans] == [
#         "inner_inner_task.task",
#         "inner_task.task",
#         "outer_task.task",
#         "some_workflow.workflow",
#     ]

#     inner_inner_task_span = spans[0]
#     inner_task_span = spans[1]
#     outer_task_span = spans[2]
#     some_workflow_span = spans[3]

#     assert (
#         inner_inner_task_span.attributes[SpanAttributes.TRACELOOP_ENTITY_PATH]
#         == "outer_task.inner_task"
#     )
#     assert (
#         inner_task_span.attributes[SpanAttributes.TRACELOOP_ENTITY_PATH] == "outer_task"
#     )
#     with raises(KeyError):
#         _ = outer_task_span.attributes[SpanAttributes.TRACELOOP_ENTITY_PATH]
#         _ = some_workflow_span.attributes[SpanAttributes.TRACELOOP_ENTITY_PATH]



import asyncio
import re
import sys
import uuid

from contextlib import contextmanager
from tqdm import tqdm
from typing import Any, Awaitable, Optional, Set, Union

from ..traceloop_sdk.instruments import Instruments
from ..traceloop_sdk.tracing.attributes import SPAN_TYPE

from .datasets import EvaluationDataset
from .laminar import Laminar as L
from .log import get_default_logger
from .types import (
    Datapoint,
    EvaluationResultDatapoint,
    EvaluatorFunction,
    ExecutorFunction,
    Numeric,
    NumericTypes,
    SpanType,
    TraceType,
)
from .utils import is_async

DEFAULT_BATCH_SIZE = 5

_evaluation = None
_set_global_evaluation = False


@contextmanager
def set_global_evaluation(set_global_evaluation: bool):
    global _set_global_evaluation
    original = _set_global_evaluation
    try:
        _set_global_evaluation = set_global_evaluation
        yield
    finally:
        _set_global_evaluation = original
        pass


def get_evaluation_url(project_id: str, evaluation_id: str):
    return f"https://www.lmnr.ai/project/{project_id}/evaluations/{evaluation_id}"


def get_average_scores(results: list[EvaluationResultDatapoint]) -> dict[str, Numeric]:
    per_score_values = {}
    for result in results:
        for key, value in result.scores.items():
            if key not in per_score_values:
                per_score_values[key] = []
            per_score_values[key].append(value)

    average_scores = {}
    for key, values in per_score_values.items():
        average_scores[key] = sum(values) / len(values)

    return average_scores


class EvaluationReporter:
    def __init__(self):
        pass

    def start(self, length: int):
        self.cli_progress = tqdm(
            total=length,
            bar_format="{bar} {percentage:3.0f}% | ETA: {remaining}s | {n_fmt}/{total_fmt}",
            ncols=60,
        )

    def update(self, batch_length: int):
        self.cli_progress.update(batch_length)

    def stopWithError(self, error: Exception):
        self.cli_progress.close()
        sys.stderr.write(f"\nError: {error}\n")

    def stop(
        self, average_scores: dict[str, Numeric], project_id: str, evaluation_id: str
    ):
        self.cli_progress.close()
        print(
            f"\nCheck the results at {get_evaluation_url(project_id, evaluation_id)}\n"
        )
        print("Average scores:")
        for name, score in average_scores.items():
            print(f"{name}: {score}")
        print("\n")


class Evaluation:
    def __init__(
        self,
        data: Union[EvaluationDataset, list[Union[Datapoint, dict]]],
        executor: Any,
        evaluators: dict[str, EvaluatorFunction],
        group_id: Optional[str] = None,
        name: Optional[str] = None,
        batch_size: int = DEFAULT_BATCH_SIZE,
        project_api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        http_port: Optional[int] = None,
        grpc_port: Optional[int] = None,
        instruments: Optional[Set[Instruments]] = None,
    ):
        """
        Initializes an instance of the Evaluations class.

        Parameters:
            data (Union[List[EvaluationDatapoint|dict], EvaluationDataset]):\
                List of data points to evaluate or an evaluation dataset.
                            `data` is the input to the executor function,
                            `target` is the input to the evaluator function.
            executor (Callable[..., Any]): The executor function.\
                            Takes the data point + any additional arguments\
                            and returns the output to evaluate.
            evaluators (dict[str, Callable[..., Any]]): Evaluator functions and\
                names. Each evaluator function takes the output of the executor\
                _and_ the target data, and returns a score. The score can be a\
                single number or a dict of string keys and number values.\
                If the score is a single number, it will be named after the\
                evaluator function. Evaluator function names must contain only\
                letters, digits, hyphens, underscores, or spaces.
            group_id (Optional[str], optional): Group id of the evaluation.
                            Defaults to "default".
            name (Optional[str], optional): The name of the evaluation.\
                            It will be auto-generated if not provided.
            batch_size (int, optional): The batch size for evaluation.
                            Defaults to DEFAULT_BATCH_SIZE.
            project_api_key (Optional[str], optional): The project API key.
                            Defaults to an empty string.
            base_url (Optional[str], optional): The base URL for Laminar API.\
                            Useful if self-hosted elsewhere. Do NOT include the\
                            port, use `http_port` and `grpc_port` instead.
                            Defaults to "https://api.lmnr.ai".
            http_port (Optional[int], optional): The port for Laminar API\
                            HTTP service. Defaults to 443 if not specified.
            grpc_port (Optional[int], optional): The port for Laminar API\
                            gRPC service. Defaults to 8443 if not specified.
            instruments (Optional[Set[Instruments]], optional): Set of modules\
                to auto-instrument. If None, all available instruments will be\
                used.
                Defaults to None.
        """

        if not evaluators:
            raise ValueError("No evaluators provided")

        # TODO: Compile regex once and then reuse it
        for evaluator_name in evaluators:
            if not re.match(r"^[\w\s-]+$", evaluator_name):
                raise ValueError(
                    f'Invalid evaluator key: "{evaluator_name}". '
                    "Keys must only contain letters, digits, hyphens,"
                    "underscores, or spaces."
                )

        self.is_finished = False
        self.reporter = EvaluationReporter()
        if isinstance(data, list):
            self.data = [
                (Datapoint.model_validate(point) if isinstance(point, dict) else point)
                for point in data
            ]
        else:
            self.data = data
        self.executor = executor
        self.evaluators = evaluators
        self.group_id = group_id
        self.name = name
        self.batch_size = batch_size
        self._logger = get_default_logger(self.__class__.__name__)
        L.initialize(
            project_api_key=project_api_key,
            base_url=base_url,
            http_port=http_port,
            grpc_port=grpc_port,
            instruments=instruments,
        )

    def run(self) -> Union[None, Awaitable[None]]:
        if self.is_finished:
            raise Exception("Evaluation is already finished")

        loop = asyncio.get_event_loop()
        if loop.is_running():
            return loop.create_task(self._run())
        else:
            return loop.run_until_complete(self._run())

    async def _run(self) -> None:
        self.reporter.start(
            len(self.data),
        )

        try:
            result_datapoints = await self._evaluate_in_batches()
        except Exception as e:
            self.reporter.stopWithError(e)
            self.is_finished = True
            return
        else:
            evaluation = L.create_evaluation(
                data=result_datapoints, group_id=self.group_id, name=self.name
            )
            average_scores = get_average_scores(result_datapoints)
            self.reporter.stop(average_scores, evaluation.projectId, evaluation.id)
            self.is_finished = True

    async def _evaluate_in_batches(self) -> list[EvaluationResultDatapoint]:
        result_datapoints = []
        for i in range(0, len(self.data), self.batch_size):
            batch = (
                self.data[i : i + self.batch_size]
                if isinstance(self.data, list)
                else self.data.slice(i, i + self.batch_size)
            )
            batch_datapoints = await self._evaluate_batch(batch)
            result_datapoints.extend(batch_datapoints)
            self.reporter.update(len(batch))
        return result_datapoints

    async def _evaluate_batch(
        self, batch: list[Datapoint]
    ) -> list[EvaluationResultDatapoint]:
        batch_promises = [self._evaluate_datapoint(datapoint) for datapoint in batch]
        results = await asyncio.gather(*batch_promises)
        return results

    async def _evaluate_datapoint(
        self, datapoint: Datapoint
    ) -> EvaluationResultDatapoint:
        with L.start_as_current_span("evaluation") as evaluation_span:
            L._set_trace_type(trace_type=TraceType.EVALUATION)
            evaluation_span.set_attribute(SPAN_TYPE, SpanType.EVALUATION.value)
            with L.start_as_current_span(
                "executor", input={"data": datapoint.data}
            ) as executor_span:
                executor_span.set_attribute(SPAN_TYPE, SpanType.EXECUTOR.value)
                output = (
                    await self.executor(datapoint.data)
                    if is_async(self.executor)
                    else self.executor(datapoint.data)
                )
                L.set_span_output(output)
            target = datapoint.target

            # Iterate over evaluators
            scores: dict[str, Numeric] = {}
            for evaluator_name, evaluator in self.evaluators.items():
                with L.start_as_current_span(
                    evaluator_name, input={"output": output, "target": target}
                ) as evaluator_span:
                    evaluator_span.set_attribute(SPAN_TYPE, SpanType.EVALUATOR.value)
                    value = (
                        await evaluator(output, target)
                        if is_async(evaluator)
                        else evaluator(output, target)
                    )
                    L.set_span_output(value)

                # If evaluator returns a single number, use evaluator name as key
                if isinstance(value, NumericTypes):
                    scores[evaluator_name] = value
                else:
                    scores.update(value)

            trace_id = uuid.UUID(int=evaluation_span.get_span_context().trace_id)
            return EvaluationResultDatapoint(
                data=datapoint.data,
                target=target,
                executor_output=output,
                scores=scores,
                trace_id=trace_id,
            )


def evaluate(
    data: Union[EvaluationDataset, list[Union[Datapoint, dict]]],
    executor: ExecutorFunction,
    evaluators: dict[str, EvaluatorFunction],
    group_id: Optional[str] = None,
    name: Optional[str] = None,
    batch_size: int = DEFAULT_BATCH_SIZE,
    project_api_key: Optional[str] = None,
    base_url: Optional[str] = None,
    http_port: Optional[int] = None,
    grpc_port: Optional[int] = None,
    instruments: Optional[Set[Instruments]] = None,
) -> Optional[Awaitable[None]]:
    """
    If added to the file which is called through `lmnr eval` command, then
    registers the evaluation; otherwise, runs the evaluation.

    If there is no event loop, creates it and runs the evaluation until
    completion.
    If there is an event loop, schedules the evaluation as a task in the
    event loop and returns an awaitable handle.

    Parameters:
        data (Union[list[EvaluationDatapoint|dict]], EvaluationDataset]):\
                    List of data points to evaluate or an evaluation dataset.
                        `data` is the input to the executor function,
                        `target` is the input to the evaluator function.
        executor (Callable[..., Any]): The executor function.\
                        Takes the data point + any additional arguments\
                        and returns the output to evaluate.
        evaluators (List[Callable[..., Any]]): 
            evaluators (dict[str, Callable[..., Any]]): Evaluator functions and\
                names. Each evaluator function takes the output of the executor\
                _and_ the target data, and returns a score. The score can be a\
                single number or a dict of string keys and number values.\
                If the score is a single number, it will be named after the\
                evaluator function. Evaluator function names must contain only\
                letters, digits, hyphens, underscores, or spaces.
        group_id (Optional[str], optional): an identifier to group evaluations.\
                        It is practical to group evaluations that evaluate\
                        the same feature on the same dataset, to be able to\
                        view their comparisons in the same place. If not\
                        provided, defaults to "default".
                        Defaults to None
        name (Optional[str], optional): Optional name of the evaluation.\
                        Used to identify the evaluation in the group.\
                        If not provided, a random name will be generated.
                        Defaults to None.
        batch_size (int, optional): The batch size for evaluation.
                        Defaults to DEFAULT_BATCH_SIZE.
        project_api_key (Optional[str], optional): The project API key.
                        Defaults to None.
        base_url (Optional[str], optional): The base URL for Laminar API.\
                        Useful if self-hosted elsewhere. Do NOT include the\
                        port, use `http_port` and `grpc_port` instead.
                        Defaults to "https://api.lmnr.ai".
        http_port (Optional[int], optional): The port for Laminar API's HTTP\
                        service. 443 is used if not specified.
                        Defaults to None.
        grpc_port (Optional[int], optional): The port for Laminar API's gRPC\
                        service. 8443 is used if not specified.
                        Defaults to None.
        instruments (Optional[Set[Instruments]], optional): Set of modules to\
                        auto-instrument. If None, all available instruments\
                        will be used.
                        Defaults to None.
    """

    evaluation = Evaluation(
        data=data,
        executor=executor,
        evaluators=evaluators,
        group_id=group_id,
        name=name,
        batch_size=batch_size,
        project_api_key=project_api_key,
        base_url=base_url,
        http_port=http_port,
        grpc_port=grpc_port,
        instruments=instruments,
    )

    global _evaluation
    if _set_global_evaluation:
        _evaluation = evaluation
    else:
        return evaluation.run()



import pkg_resources

installed_packages = {p.key for p in pkg_resources.working_set}


def is_package_installed(package_name: str) -> bool:
    # return importlib.util.find_spec(package_name) is not None
    return package_name in installed_packages



import json
from functools import wraps
import logging
import os
import pydantic
import types
from typing import Any, Optional

from opentelemetry import trace
from opentelemetry import context as context_api

from lmnr.sdk.utils import get_input_from_func_args, is_method
from lmnr.traceloop_sdk.tracing import get_tracer
from lmnr.traceloop_sdk.tracing.attributes import SPAN_INPUT, SPAN_OUTPUT, SPAN_PATH
from lmnr.traceloop_sdk.tracing.tracing import TracerWrapper, get_span_path
from lmnr.traceloop_sdk.utils.json_encoder import JSONEncoder


class CustomJSONEncoder(JSONEncoder):
    def default(self, o: Any) -> Any:
        if isinstance(o, pydantic.BaseModel):
            return o.model_dump_json()
        try:
            return super().default(o)
        except TypeError:
            return str(o)  # Fallback to string representation for unsupported types


def json_dumps(data: dict) -> str:
    try:
        return json.dumps(data, cls=CustomJSONEncoder)
    except Exception:
        # Log the exception and return a placeholder if serialization completely fails
        logging.warning("Failed to serialize data to JSON, type: %s", type(data))
        return "{}"  # Return an empty JSON object as a fallback


def entity_method(
    name: Optional[str] = None,
):
    def decorate(fn):
        @wraps(fn)
        def wrap(*args, **kwargs):
            if not TracerWrapper.verify_initialized():
                return fn(*args, **kwargs)

            span_name = name or fn.__name__

            with get_tracer() as tracer:
                span = tracer.start_span(span_name)

                span_path = get_span_path(span_name)
                span.set_attribute(SPAN_PATH, span_path)
                ctx = context_api.set_value("span_path", span_path)

                ctx = trace.set_span_in_context(span, ctx)
                ctx_token = context_api.attach(ctx)

                try:
                    if _should_send_prompts():
                        span.set_attribute(
                            SPAN_INPUT,
                            json_dumps(
                                get_input_from_func_args(
                                    fn, is_method(fn), args, kwargs
                                )
                            ),
                        )
                except TypeError:
                    pass

                res = fn(*args, **kwargs)

                # span will be ended in the generator
                if isinstance(res, types.GeneratorType):
                    return _handle_generator(span, res)

                try:
                    if _should_send_prompts():
                        span.set_attribute(
                            SPAN_OUTPUT,
                            json_dumps(res),
                        )
                except TypeError:
                    pass

                span.end()
                context_api.detach(ctx_token)

                return res

        return wrap

    return decorate


# Async Decorators


def aentity_method(
    name: Optional[str] = None,
):
    def decorate(fn):
        @wraps(fn)
        async def wrap(*args, **kwargs):
            if not TracerWrapper.verify_initialized():
                return await fn(*args, **kwargs)

            span_name = name or fn.__name__

            with get_tracer() as tracer:
                span = tracer.start_span(span_name)

                span_path = get_span_path(span_name)
                span.set_attribute(SPAN_PATH, span_path)
                ctx = context_api.set_value("span_path", span_path)

                ctx = trace.set_span_in_context(span, ctx)
                ctx_token = context_api.attach(ctx)

                try:
                    if _should_send_prompts():
                        span.set_attribute(
                            SPAN_INPUT,
                            json_dumps(
                                get_input_from_func_args(
                                    fn, is_method(fn), args, kwargs
                                )
                            ),
                        )
                except TypeError:
                    pass

                res = await fn(*args, **kwargs)

                # span will be ended in the generator
                if isinstance(res, types.AsyncGeneratorType):
                    return await _ahandle_generator(span, ctx_token, res)

                try:
                    if _should_send_prompts():
                        span.set_attribute(SPAN_OUTPUT, json_dumps(res))
                except TypeError:
                    pass

                span.end()
                context_api.detach(ctx_token)

                return res

        return wrap

    return decorate


def _handle_generator(span, res):
    # for some reason the SPAN_KEY is not being set in the context of the generator, so we re-set it
    context_api.attach(trace.set_span_in_context(span))
    yield from res

    span.end()

    # Note: we don't detach the context here as this fails in some situations
    # https://github.com/open-telemetry/opentelemetry-python/issues/2606
    # This is not a problem since the context will be detached automatically during garbage collection


async def _ahandle_generator(span, ctx_token, res):
    async for part in res:
        yield part

    span.end()
    context_api.detach(ctx_token)


def _should_send_prompts():
    return (
        os.getenv("TRACELOOP_TRACE_CONTENT") or "true"
    ).lower() == "true" or context_api.get_value("override_enable_content_tracing")



import os
import sys
from pathlib import Path

from typing import Optional, Set
from opentelemetry.sdk.trace import SpanProcessor
from opentelemetry.sdk.trace.export import SpanExporter
from opentelemetry.sdk.resources import SERVICE_NAME
from opentelemetry.propagators.textmap import TextMapPropagator
from opentelemetry.util.re import parse_env_headers

from lmnr.traceloop_sdk.instruments import Instruments
from lmnr.traceloop_sdk.config import (
    is_content_tracing_enabled,
    is_tracing_enabled,
)
from lmnr.traceloop_sdk.tracing.tracing import TracerWrapper
from typing import Dict


class Traceloop:
    AUTO_CREATED_KEY_PATH = str(
        Path.home() / ".cache" / "traceloop" / "auto_created_key"
    )
    AUTO_CREATED_URL = str(Path.home() / ".cache" / "traceloop" / "auto_created_url")

    __tracer_wrapper: TracerWrapper

    @staticmethod
    def init(
        app_name: Optional[str] = sys.argv[0],
        api_endpoint: str = "https://api.lmnr.ai",
        api_key: Optional[str] = None,
        headers: Dict[str, str] = {},
        disable_batch=False,
        exporter: Optional[SpanExporter] = None,
        processor: Optional[SpanProcessor] = None,
        propagator: Optional[TextMapPropagator] = None,
        should_enrich_metrics: bool = True,
        resource_attributes: dict = {},
        instruments: Optional[Set[Instruments]] = None,
    ) -> None:
        api_endpoint = os.getenv("TRACELOOP_BASE_URL") or api_endpoint
        api_key = os.getenv("TRACELOOP_API_KEY") or api_key

        if not is_tracing_enabled():
            # print(Fore.YELLOW + "Tracing is disabled" + Fore.RESET)
            return

        enable_content_tracing = is_content_tracing_enabled()

        headers = os.getenv("TRACELOOP_HEADERS") or headers

        if isinstance(headers, str):
            headers = parse_env_headers(headers)

        if (
            not exporter
            and not processor
            and api_endpoint == "https://api.lmnr.ai"
            and not api_key
        ):
            print(
                "Error: Missing API key,"
                + " go to project settings to create one"
            )
            print("Set the LMNR_PROJECT_API_KEY environment variable to the key")
            return

        if api_key and not exporter and not processor and not headers:
            headers = {
                "Authorization": f"Bearer {api_key}",
            }

        # print(Fore.RESET)

        # Tracer init
        resource_attributes.update({SERVICE_NAME: app_name})
        TracerWrapper.set_static_params(
            resource_attributes, enable_content_tracing, api_endpoint, headers
        )
        Traceloop.__tracer_wrapper = TracerWrapper(
            disable_batch=disable_batch,
            processor=processor,
            propagator=propagator,
            exporter=exporter,
            should_enrich_metrics=should_enrich_metrics,
            instruments=instruments,
        )
